angular.module("openITCOCKPIT").controller("OpenstreetmapIndexController", function ($scope, $q, $http, $timeout, $interval) {
    $scope.mapRefIntervalSeconds = 15000;
    $scope.mapMinimumIntervalSeconds = 15000;
    $scope.mapURL = "";
    $scope.parametersMissing = null;
    $scope.mapViewOptionsHasBeenChanged = false;
    $scope.locations = [];
    $scope.allLocations = [];
    /*** Filter Settings ***/
    var defaultFilter = function defaultFilter() {
        $scope.included = {
            Location: {
                id: []
            }
        };
    };
    $scope.init = false;
    $scope.mapOptions = {
        center: {
            lat: 0,
            lng: 0
        },
        zoom: 0,
        fullscreenControl: true
    };

    $scope.map = L.map("map", $scope.mapOptions);

    var intervalId = null;

    $scope.loadLocationsInMap = function (includeLocations) {
        if (includeLocations === true) {
            $scope.included.Location.id = [];
            /**
             * Filter locations if out of view
             */
            var bounds = $scope.map.getBounds();
            for (key in $scope.allLocations) {
                if (bounds.contains(new L.LatLng($scope.allLocations[key].Location.latitude, $scope.allLocations[key].Location.longitude))) {
                    $scope.included.Location.id.push($scope.allLocations[key].Location.id);
                }
            }
            if ($scope.included.Location.id.length === 0) {
                return;
            }
        }
        $q.all([$http.get("/openstreetmap_module/openstreetmap/getAclAndSettings.json", {
            params: {
                "angular": true
            }
        }), $http.get("/openstreetmap_module/openstreetmap/index.json", {
            params: {
                "angular": true,
                'includedLocationId[]': $scope.included.Location.id
            }
        })]).then(function (results) {
            $scope.acl = results[0].data.acl;
            $scope.mapRefIntervalSeconds = parseInt(results[0].data.settings.reload_interval, 10) * 1000; // * 1000 in milliseconds
            $scope.mapURL = results[0].data.settings.server_url;
            $scope.titleResetButton = results[0].data.settings.titleResetButton;

            if ($scope.init === false) {
                $scope.allLocations = results[1].data.locations;
                $scope.locationPoints = results[1].data.locationPoints;
            }
            $scope.locations = results[1].data.locations;
            $scope.parametersMissing = true;
            if ($scope.mapURL && $scope.locations.length > 0) {
                $scope.parametersMissing = false;
                loadMap(results[1].data.locationPoints);
                $scope.init = true;
            }
            if (!$scope.parametersMissing && $scope.mapRefIntervalSeconds >= $scope.mapMinimumIntervalSeconds) {
                // minimum seconds for refresh
                if (intervalId === null) {
                    intervalId = $interval(function () {
                        $scope.loadLocationsInMap(true);
                    }, $scope.mapRefIntervalSeconds);
                }
            }
        });
    };

    defaultFilter();
    $scope.loadLocationsInMap(false);

    var hasChanges = function hasChanges(mapOptions) {
        if (mapOptions.zoom === 0) {
            return false;
        }
        if (mapOptions.zoom !== $scope.map.getZoom()) {
            return true;
        }
        if (mapOptions.center.lat !== $scope.map.getCenter()['lat'] && mapOptions.center.lng !== $scope.map.getCenter()['lng']) {
            return true;
        }
        return false;
    };

    var newCustomControl = L.Control.extend({
        options: {
            position: "topleft"
        },
        onAdd: function onAdd() {
            var container = L.DomUtil.create("div", "leaflet-bar leaflet-control leaflet-control-custom");
            container.title = $scope.titleResetButton;
            container.onclick = function () {
                $scope.mapOptions.zoom = 0;
                defaultFilter();
                $scope.loadLocationsInMap(false);
            };
            return container;
        }
    });

    var loadMap = function loadMap(locationPoints) {
        if (typeof locationPoints === "undefined") {
            return;
        }
        if (!hasChanges($scope.mapOptions)) {
            $scope.map.fitBounds(new L.LatLngBounds(locationPoints));
            $scope.mapOptions.zoom = $scope.map.getZoom();
            $scope.mapOptions.center = $scope.map.getCenter();
        }
        L.tileLayer($scope.mapURL, {
            maxNativeZoom: 18,
            maxZoom: 20,
            attribution: "&copy; <a target='_blank' href='http://openstreetmap.org/copyright'>OpenStreetMap</a> contributors | <a target='_blank' href='https://it-novum.com'>it-novum.com</a>",
            noWrap: false
        }).addTo($scope.map);

        var options = {
            click: function click(data) {
                if ($scope.map.isFullscreen() === true) {
                    return;
                }
                var containerIds = [];
                data.forEach(function (point) {
                    containerIds.push(point.o.id);
                });
                $scope.showOpenstreetMapSummaryState(containerIds, $scope.acl);
            }
        };

        var cells = _.chain($scope.locations).groupBy(function (location) {
            return location.Container.id;
        }).map(function (values) {
            return {
                latitude: Number(values[0].Location.latitude),
                longitude: Number(values[0].Location.longitude),
                id: values[0].Container.id,
                name: values[0].Container.name,
                colorcode: values[0].Location.colorcode
            };
        }).value();

        L.hexbinLayer(options).addTo($scope.map).data(cells);
        if ($scope.init === false) {
            $scope.map.addControl(new newCustomControl());
        }
    };
});